package com.jjt.biz.service.impl;

import com.jjt.biz.domain.AlarmRecord;
import com.jjt.biz.domain.BizObjMetrics;
import com.jjt.biz.mapper.AlarmRecordMapper;
import com.jjt.biz.service.IAlarmRecordService;
import com.jjt.common.utils.DateUtils;
import com.jjt.common.utils.IntervalUtil;
import com.jjt.common.utils.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 告警记录Service业务层处理
 *
 * @author jjt
 * @date 2024-08-08
 */
@Service
public class AlarmRecordServiceImpl implements IAlarmRecordService {
    @Resource
    private AlarmRecordMapper alarmRecordMapper;

    /**
     * 查询告警记录
     *
     * @param alarmId 告警记录主键
     * @return 告警记录
     */
    @Override
    public AlarmRecord selectAlarmRecordByAlarmId(Long alarmId) {
        return alarmRecordMapper.selectAlarmRecordByAlarmId(alarmId);
    }

    /**
     * 查询告警记录列表
     *
     * @param alarmRecord 告警记录
     * @return 告警记录
     */
    @Override
    public List<AlarmRecord> selectAlarmRecordList(AlarmRecord alarmRecord) {
        return alarmRecordMapper.selectAlarmRecordList(alarmRecord);
    }

    /**
     * 新增告警记录
     *
     * @param alarmRecord 告警记录
     * @return 结果
     */
    @Override
    public int insertAlarmRecord(AlarmRecord alarmRecord) {
        alarmRecord.setCreateTime(DateUtils.getNowDate());
        return alarmRecordMapper.insertAlarmRecord(alarmRecord);
    }

    /**
     * 修改告警记录
     *
     * @param alarmRecord 告警记录
     * @return 结果
     */
    @Override
    public int updateAlarmRecord(AlarmRecord alarmRecord) {
        alarmRecord.setUpdateTime(DateUtils.getNowDate());
        return alarmRecordMapper.updateAlarmRecord(alarmRecord);
    }

    /**
     * 批量删除告警记录
     *
     * @param alarmIds 需要删除的告警记录主键
     * @return 结果
     */
    @Override
    public int deleteAlarmRecordByAlarmIds(Long[] alarmIds) {
        return alarmRecordMapper.deleteAlarmRecordByAlarmIds(alarmIds);
    }

    /**
     * 删除告警记录信息
     *
     * @param alarmId 告警记录主键
     * @return 结果
     */
    @Override
    public int deleteAlarmRecordByAlarmId(Long alarmId) {
        return alarmRecordMapper.deleteAlarmRecordByAlarmId(alarmId);
    }

    /**
     * 告警等级排行
     *
     * @param alarmRecord 对象
     * @return 结果
     */
    @Override
    public List<Map<String, Object>> levelSort(AlarmRecord alarmRecord) {
        return alarmRecordMapper.levelSort(alarmRecord);
    }

    /**
     * 按小时告警趋势
     *
     * @param alarmRecord 对象
     * @return 结果
     */
    @Override
    public List<Map<String, Object>> listHours(AlarmRecord alarmRecord) {
        return alarmRecordMapper.listHours(alarmRecord);
    }

    /**
     * 处理告警和事件
     *
     * @param om        指标对象
     * @param recordMap 正在告警的列表
     * @param date      时间
     * @return 结果
     */
    @Override
    public Map<String, AlarmRecord> process(BizObjMetrics om, Map<Long, AlarmRecord> recordMap, Date date) {
        Map<String, AlarmRecord> result = new HashMap<>(16);
        // 需要查询是否已经有告警记录
        // 1.当前没有告警，但之前有未结束的告警。
        // 2.如果该指标有告警，且告警等级相同，则不用管
        // 3.如果该指标有告警，但告警等级不同，则上一次告警结束，新增一条告警。
        // 4.如果之前没有告警，当前告警，则新增告警。
        AlarmRecord record = recordMap.get(om.getObjMetricsId());
        String alarmLevel = getAlarmLevel(om.getDValue().floatValue(), om);
        if (alarmLevel == null) {
            //如果没有告警，且之前有告警记录，则结束之前告警
            if (record != null) {
                record.setAlarmType("2");
                record.setEndTime(date);
                result.put("update", record);
//                updateAlarmRecord(record);
            }
            return result;
        }

        if (record != null) {
            if (alarmLevel.equals(record.getAlarmLevel())) {
                //如果有相同告警，则本次不处理
                return result;
            } else {
                //有告警记录，但告警等级不一样
                //结束上一次告警
                record.setAlarmType("2");
                record.setEndTime(date);
                result.put("update", record);

//                updateAlarmRecord(record);
            }
        }
        record = new AlarmRecord();
        record.setObjId(om.getObjId());
        record.setObjMetricsId(om.getObjMetricsId());
        record.setAlarmLevel(alarmLevel);
        record.setAlarmValue(om.getDValue());
        record.setAlarmType("1");
        record.setAlarmTime(date);
        record.setCreateTime(new Date());
        result.put("add", record);
        return result;
//        insertAlarmRecord(record);
    }

    /**
     * 通过对象ID查询当前正在告警的列表,并转换成map
     *
     * @param objId 对象ID
     * @return 结果
     */
    @Override
    public Map<Long, AlarmRecord> selectAlarmRecordListCurr(Long objId) {
        AlarmRecord search = new AlarmRecord();
        search.setObjId(objId);
        search.setAlarmType("1");
        List<AlarmRecord> list = selectAlarmRecordList(search);
        return list.stream().collect(Collectors.toMap(AlarmRecord::getObjMetricsId, Function.identity()));
    }

    /**
     * 查询是否正在告警？
     *
     * @param metricsId 指标对象
     * @return 结果
     */
//    private AlarmRecord exists(Long metricsId) {
//        AlarmRecord search = new AlarmRecord();
//        search.setObjMetricsId(metricsId);
//        search.setAlarmType("1");
//
//        List<AlarmRecord> list = selectAlarmRecordList(search);
//        if (list.size() > 0) {
//            return list.get(0);
//        } else {
//            return null;
//        }
//    }

    /**
     * 获取告警等级
     *
     * @param value         值
     * @param bizObjMetrics 指标对象
     * @return
     */
    private String getAlarmLevel(Float value, BizObjMetrics bizObjMetrics) {
        if (StringUtils.isNotEmpty(bizObjMetrics.getAlarmLow()) && IntervalUtil.inNumRange(value, bizObjMetrics.getAlarmLow())) {
            return "low";
        } else if (StringUtils.isNotEmpty(bizObjMetrics.getAlarmMid()) && IntervalUtil.inNumRange(value, bizObjMetrics.getAlarmMid())) {
            return "mid";
        } else if (StringUtils.isNotEmpty(bizObjMetrics.getAlarmHigh()) && IntervalUtil.inNumRange(value, bizObjMetrics.getAlarmHigh())) {
            return "high";
        }
        return null;
    }


}
